# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
from urllib2 import URLError, HTTPError
'''
Created on May 10, 2011

@author: evan
'''

from kayako.core.lib import UnsetParameter
from kayako.core.object import KayakoObject
from kayako.exception import KayakoRequestError, KayakoResponseError
from lxml import etree
import base64



__all__ = [
    'Ticket',
    'TicketAttachment',
    'TicketNote',
    'TicketPost',
    'TicketPriority',
    'TicketStatus',
    'TicketType',
]

class Ticket(KayakoObject):
    '''
    Kayako Ticket API Object.
    
    subject          The Ticket Subject
    fullname         Full Name of creator
    email            Email Address of creator
    contents         The contents of the first ticket post
    departmentid     The Department ID
    ticketstatusid   The Ticket Status ID
    ticketpriorityid The Ticket Priority ID
    tickettypeid     The Ticket Type ID
    userid           The User ID, if the ticket is to be created as a user.
    staffid          The Staff ID, if the ticket is to be created as a staff
    ownerstaffid     The Owner Staff ID, if you want to set an Owner for this ticket
    type             The ticket type: 'default' or 'phone' 
    '''

    controller = '/Tickets/Ticket'

    __parameters__ = [
        'id',
        'subject',
        'fullname',
        'email',
        'contents',
        'departmentid',
        'ticketstatusid',
        'ticketpriorityid', # synonym for priorityid
        'tickettypeid',
        'userid',
        'staffid',
        'ownerstaffid',
        'type',
        'flagtype',
        'displayid',
        'statusid',
        'typeid',
        'userorganization',
        'userorganizationid',
        'ownerstaffname',
        'lastreplier',
        'creationtime',
        'lastactivity',
        'laststaffreply',
        'lastuserreply',
        'slaplanid',
        'nextreplydue',
        'resolutiondue',
        'replies',
        'ipaddress',
        'creator',
        'creationmode',
        'creationtype',
        'isescalated',
        'escalationruleid',
        'tags',
        'watchers',
        'workflows',
        'notes',
        'posts',
    ]

    __required_add_parameters__ = ['subject', 'fullname', 'email', 'contents', 'departmentid', 'ticketstatusid', 'ticketpriorityid', 'tickettypeid', ]
    __add_parameters__ = ['subject', 'fullname', 'email', 'contents', 'departmentid', 'ticketstatusid', 'ticketpriorityid', 'tickettypeid', 'userid', 'staffid', 'ownerstaffid', 'type']

    __required_save_parameters__ = []
    __save_parameters__ = ['subject', 'fullname', 'email', 'departmentid', 'ticketstatusid', 'ticketpriorityid', 'ownerstaffid', 'userid', ]

    @classmethod
    def _parse_ticket(cls, api, ticket_tree):

        ticketid = cls._parse_int(ticket_tree.get('id'))

        workflows = [dict(id=workflow_node.get('id'), title=workflow_node.get('title')) for workflow_node in ticket_tree.findall('workflow')]
        watchers = [dict(staffid=watcher_node.get('staffid'), name=watcher_node.get('name')) for watcher_node in ticket_tree.findall('watcher')]
        notes = [TicketNote(api, **TicketNote._parse_ticket_note(ticket_note_tree, ticketid)) for ticket_note_tree in ticket_tree.findall('note')]

        posts = []
        posts_node = ticket_tree.find('posts')
        if posts_node is not None:
            posts = [TicketPost(api, **TicketPost._parse_ticket_post(ticket_post_tree, ticketid)) for ticket_post_tree in posts_node.findall('post')]

        params = dict(
            id=ticketid,
            subject=cls._get_string(ticket_tree.find('subject')),
            fullname=cls._get_string(ticket_tree.find('fullname')),
            email=cls._get_string(ticket_tree.find('email')),
            departmentid=cls._get_int(ticket_tree.find('departmentid')),
            ticketstatusid=cls._get_int(ticket_tree.find('ticketstatusid'), required=False, strict=False),
            ticketpriorityid=cls._get_int(ticket_tree.find('priorityid')), # Note the difference, request param is ticketpriorityid, response is priorityid
            tickettypeid=cls._get_int(ticket_tree.find('tickettypeid'), required=False, strict=False),
            userid=cls._get_int(ticket_tree.find('userid')),
            ownerstaffid=cls._get_int(ticket_tree.find('ownerstaffid')),
            flagtype=cls._parse_int(ticket_tree.get('flagtype'), 'flagtype'),
            displayid=cls._get_string(ticket_tree.find('displayid')),
            statusid=cls._get_int(ticket_tree.find('statusid')),
            typeid=cls._get_int(ticket_tree.find('typeid')),
            userorganization=cls._get_string(ticket_tree.find('userorganization')),
            userorganizationid=cls._get_int(ticket_tree.find('userorganizationid'), required=False),
            ownerstaffname=cls._get_string(ticket_tree.find('ownerstaffname')),
            lastreplier=cls._get_string(ticket_tree.find('lastreplier')),
            creationtime=cls._get_date(ticket_tree.find('creationtime')),
            lastactivity=cls._get_date(ticket_tree.find('lastactivity')),
            laststaffreply=cls._get_date(ticket_tree.find('laststaffreply')),
            lastuserreply=cls._get_date(ticket_tree.find('lastuserreply')),
            slaplanid=cls._get_int(ticket_tree.find('slaplanid')),
            nextreplydue=cls._get_date(ticket_tree.find('nextreplydue')),
            resolutiondue=cls._get_date(ticket_tree.find('resolutiondue')),
            replies=cls._get_int(ticket_tree.find('replies')),
            ipaddress=cls._get_string(ticket_tree.find('ipaddress')),
            creator=cls._get_int(ticket_tree.find('creator')),
            creationmode=cls._get_int(ticket_tree.find('creationmode')),
            creationtype=cls._get_int(ticket_tree.find('creationtype')),
            isescalated=cls._get_boolean(ticket_tree.find('isescalated')),
            escalationruleid=cls._get_int(ticket_tree.find('escalationruleid')),
            tags=cls._get_string(ticket_tree.find('tags')),
            watchers=watchers,
            workflows=workflows,
            notes=notes,
            posts=posts,
        )
        return params

    @classmethod
    def get_all(cls, api, departmentid, ticketstatusid= -1, ownerstaffid= -1, userid= -1):
        '''
        Get all of the tickets filtered by the parameters:
        Lists are converted to comma-separated values.
        Required:
            departmentid     Filter the tickets by the specified department id, you can specify multiple id's by separating the values using a comma. Example: 1,2,3
        Optional:
            ticketstatusid   Filter the tickets by the specified ticket status id, you can specify multiple id's by separating the values using a comma. Example: 1,2,3
            ownerstaffid     Filter the tickets by the specified owner staff id, you can specify multiple id's by separating the values using a comma. Example: 1,2,3
            userid           Filter the tickets by the specified user id, you can specify multiple id's by separating the values using a comma. Example: 1,2,3 
        '''

        if isinstance(departmentid, (list, tuple)):
            departmentid = ','.join([str(id_item) for id_item in departmentid])
        if isinstance(ticketstatusid, (list, tuple)):
            ticketstatusid = ','.join([str(id_item) for id_item in ticketstatusid])
        if isinstance(ownerstaffid, (list, tuple)):
            ownerstaffid = ','.join([str(id_item) for id_item in ownerstaffid])
        if isinstance(userid, (list, tuple)):
            userid = ','.join([str(id_item) for id_item in userid])

        response = api._request('%s/ListAll/%s/%s/%s/%s/' % (cls.controller, departmentid, ticketstatusid, ownerstaffid, userid), 'GET')
        tree = etree.parse(response)
        return [Ticket(api, **cls._parse_ticket(api, ticket_tree)) for ticket_tree in tree.findall('ticket')]

    @classmethod
    def get(cls, api, id):
        try:
            response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        except KayakoResponseError, error:
            if len(error.args) and isinstance(error.args[0], (URLError, HTTPError)) and error.args[0].getcode() == 404:
                return None
        tree = etree.parse(response)
        node = tree.find('ticket')
        if node is None:
            return None
        params = cls._parse_ticket(api, node)
        return Ticket(api, **params)

    def add(self):
        '''
        Add this Ticket.
        
        Requires:
            subject          The Ticket Subject
            fullname         Full Name of creator
            email            Email Address of creator
            contents         The contents of the first ticket post
            departmentid     The Department ID
            ticketstatusid   The Ticket Status ID
            ticketpriorityid The Ticket Priority ID
            tickettypeid     The Ticket Type ID
        One of these must be present:
            userid           The User ID, if the ticket is to be created as a user.
            staffid          The Staff ID, if the ticket is to be created as a staff
        Optional:
            ownerstaffid     The Owner Staff ID, if you want to set an Owner for this ticket
            type             The ticket type: 'default' or 'phone'
        '''
        if self.id is not UnsetParameter:
            raise KayakoRequestError('Cannot add a pre-existing %s. Use save instead. (id: %s)' % (self.__class__.__name__, self.id))

        parameters = self.add_parameters
        for required_parameter in self.__required_add_parameters__:
            if required_parameter not in parameters:
                raise KayakoRequestError('Cannot add %s: Missing required field: %s.' % (self.__class__.__name__, required_parameter))

        if ('userid' not in parameters and 'staffid' not in parameters) or ('userid' in parameters and 'staffid' in parameters):
            raise KayakoRequestError('To add a Ticket, just one of the following parameters must be set: userid, staffid. (id: %s)' % self.id)

        response = self.api._request(self.controller, 'POST', **parameters)
        tree = etree.parse(response)
        self.id = int(tree.find('ticket').get('id'))

    def save(self):
        '''
        Save this ticket.
        
        Saves only the following:
            subject          The Ticket Subject
            fullname         Full Name of creator
            email            Email Address of creator
            departmentid     The Department ID
            ticketstatusid   The Ticket Status ID
            ticketpriorityid The Ticket Priority ID
            tickettypeid     The Ticket Type ID
            ownerstaffid     The Owner Staff ID, if you want to set an Owner for this ticket
            userid           The User ID, if you want to change the user for this ticket 
        '''
        self._save('%s/%s/' % (self.controller, self.id))

    def delete(self):
        self._delete('%s/%s/' % (self.controller, self.id))

    def __str__(self):
        return '<Ticket (%s): %s>' % (self.id, self.subject)

class TicketAttachment(KayakoObject):
    '''
    Kayako TicketAttachment API Object.
    
    ticketid     The unique numeric identifier of the ticket.
    ticketpostid The unique numeric identifier of the ticket post.
    filename     The file name for the attachment
    contents     The BASE64 encoded attachment contents 
    filesize
    filetype
    dateline
    '''

    controller = '/Tickets/TicketAttachment'

    __parameters__ = [
        'id',
        'ticketid',
        'ticketpostid',
        'filename',
        'filesize',
        'filetype',
        'contents',
        'dateline',
    ]

    __required_add_parameters__ = ['ticketid', 'ticketpostid', 'filename', 'contents']
    __add_parameters__ = ['ticketid', 'ticketpostid', 'filename', 'contents']

    __required_save_parameters__ = []
    ''' Save not available for TicketAttachment. '''
    __save_parameters__ = []
    ''' Save not available for TicketAttachment. '''


    @classmethod
    def _parse_ticket_attachment(cls, ticket_attachment_tree):

        params = dict(
            id=cls._get_int(ticket_attachment_tree.find('id')),
            ticketid=cls._get_int(ticket_attachment_tree.find('ticketid')),
            ticketpostid=cls._get_int(ticket_attachment_tree.find('ticketpostid')),
            filename=cls._get_string(ticket_attachment_tree.find('filename')),
            filesize=cls._get_int(ticket_attachment_tree.find('filesize')),
            filetype=cls._get_string(ticket_attachment_tree.find('filetype')),
            contents=cls._get_string(ticket_attachment_tree.find('contents')),
            dateline=cls._get_date(ticket_attachment_tree.find('dateline')),
        )
        return params

    @classmethod
    def get_all(cls, api, ticketid):
        '''
        Get all of the TicketAttachments for a ticket.
        Required:
            ticketid     The unique numeric identifier of the ticket. 
        '''
        response = api._request('%s/ListAll/%s' % (cls.controller, ticketid), 'GET')
        tree = etree.parse(response)
        return [TicketAttachment(api, **cls._parse_ticket_attachment(ticket_attachment_tree)) for ticket_attachment_tree in tree.findall('attachment')]

    @classmethod
    def get(cls, api, ticketid, attachmentid):
        try:
            response = api._request('%s/%s/%s/' % (cls.controller, ticketid, attachmentid), 'GET')
        except KayakoResponseError, error:
            if len(error.args) and isinstance(error.args[0], (URLError, HTTPError)) and error.args[0].getcode() == 404:
                return None
        tree = etree.parse(response)
        node = tree.find('attachment')
        if node is None:
            return None
        params = cls._parse_ticket_attachment(node)
        return TicketAttachment(api, **params)

    def add(self):
        '''
        Add this TicketAttachment.
        
        Requires:
            ticketid     The unique numeric identifier of the ticket.
            ticketpostid The unique numeric identifier of the ticket post.
            filename     The file name for the attachment
            contents     The BASE64 encoded attachment contents 
        '''
        response = self._add(self.controller)
        tree = etree.parse(response)
        self.id = int(tree.find('attachment').find('id').text)

    def delete(self):
        if self.ticketid is None or self.ticketid is UnsetParameter:
            raise KayakoRequestError('Cannot delete a TicketAttachment without being attached to a ticket. The ID of the Ticket (ticketid) has not been specified.')
        self._delete('%s/%s/%s/' % (self.controller, self.ticketid, self.id))

    def get_contents(self):
        ''' Return the unencoded contents of this TicketAttachment. '''
        if self.contents:
            return base64.b64decode(self.contents)

    def set_contents(self, contents):
        ''' 
        Set this TicketAttachment's contents to Base 64 encoded data, or set the
        contents to nothing.
        '''
        if contents:
            self.contents = base64.b64encode(contents)
        else:
            self.contents = None

    def __str__(self):
        return '<TicketAttachment (%s): %s>' % (self.id, self.filename)

class TicketNote(KayakoObject):
    '''
    Kayako TicketNote API Object.
    
    ticketid     The unique numeric identifier of the ticket.
    contents     The ticket note contents
    staffid      The Staff ID, if the ticket is to be created as a staff.
    fullname     The Fullname, if the ticket is to be created without providing a staff user. Example: System messages, Alerts etc.
    forstaffid   The Staff ID, this value can be provided if you wish to restrict the note visibility to a specific staff
    notecolor    The Note Color, for more information see note colors (http://wiki.kayako.com/display/DEV/Mobile+-+Constants)
    '''

    controller = '/Tickets/TicketNote'

    __parameters__ = [
        'id',
        'ticketid',
        'contents',
        'staffid',
        'fullname',
        'forstaffid',
        'notecolor',
        'type',
        'creatorstaffid',
        'creatorstaffname',
        'creationdate',
    ]

    __required_add_parameters__ = ['ticketid', 'contents']
    __add_parameters__ = ['ticketid', 'contents', 'staffid', 'fullname', 'forstaffid', 'notecolor']

    __required_save_parameters__ = []
    ''' Save not available for TicketNote. '''
    __save_parameters__ = []
    ''' Save not available for TicketNote. '''

    @classmethod
    def _parse_ticket_note(cls, ticket_note_tree, ticketid):
        params = dict(
            ticketid=ticketid,
            contents=ticket_note_tree.text,
            staffid=cls._parse_int(ticket_note_tree.get('staffid'), required=False),
            forstaffid=cls._parse_int(ticket_note_tree.get('forstaffid')),
            notecolor=cls._parse_int(ticket_note_tree.get('notecolor')),
            type=ticket_note_tree.get('type'),
            creatorstaffid=cls._parse_int(ticket_note_tree.get('creatorstaffid')),
            creatorstaffname=ticket_note_tree.get('creatorstaffname'),
            creationdate=cls._parse_date(ticket_note_tree.get('creationdate')),
        )
        return params

    @classmethod
    def get_all(cls, api, ticketid):
        '''
        Get all of the TicketNotes for a ticket.
        Required:
            ticketid     The unique numeric identifier of the ticket. 
        '''
        response = api._request('%s/ListAll/%s' % (cls.controller, ticketid), 'GET')
        tree = etree.parse(response)
        return [TicketNote(api, **cls._parse_ticket_note(ticket_note_tree, ticketid)) for ticket_note_tree in tree.findall('note')]

#    @classmethod
#    def get(cls, api, ticketid, id):
#        response = api._request('%s/%s/%s/' % (cls.controller, ticketid, id), 'GET')
#        tree = etree.parse(response)
#        node = tree.find('note')
#        if node is None:
#            return None
#        params = cls._parse_ticket_note(node, ticketid)
#        return TicketNote(api, **params)

    def add(self):
        '''
        Add this TicketNote.
        
        Requires:
            ticketid     The unique numeric identifier of the ticket.
            contents     The ticket note contents
        Requires one:
            staffid      The Staff ID, if the ticket is to be created as a staff.
            fullname     The Fullname, if the ticket is to be created without providing a staff user. Example: System messages, Alerts etc.
        Optional:
            forstaffid   The Staff ID, this value can be provided if you wish to restrict the note visibility to a specific staff
            notecolor    The Note Color, for more information see note colors
        '''
        if self.id is not UnsetParameter:
            raise KayakoRequestError('Cannot add a pre-existing %s. Use save instead. (id: %s)' % (self.__class__.__name__, self.id))

        parameters = self.add_parameters

        for required_parameter in self.__required_add_parameters__:
            if required_parameter not in parameters:
                raise KayakoRequestError('Cannot add %s: Missing required field: %s.' % (self.__class__.__name__, required_parameter))

        if ('fullname' not in parameters and 'staffid' not in parameters) or ('fullname' in parameters and 'staffid' in parameters):
            raise KayakoRequestError('To add a TicketNote, just one of the following parameters must be set: fullname, staffid. (id: %s)' % self.id)

        #response = self.api._request(self.controller, 'POST', **parameters)
        self.api._request(self.controller, 'POST', **parameters)
        #tree = etree.parse(response)
        #self.id = int(tree.find('note').get('id'))

#    def delete(self):
#        if self.ticketid is None or self.ticketid is UnsetParameter:
#            raise KayakoRequestError('Cannot delete a TicketNote without being attached to a ticket. The ID of the Ticket (ticketid) has not been specified.')
#        self._delete('%s/%s/%s/' % (self.controller, self.ticketid, self.id))

    def __str__(self):
        return '<TicketNote (%s): %s>' % (self.id, self.contents[:20])

class TicketPost(KayakoObject):
    '''
    Kayako TicketPost API Object.
    
    ticketid   The unique numeric identifier of the ticket.
    subject    The ticket post subject
    contents   The ticket post contents
    userid     The User ID, if the ticket post is to be created as a user.
    staffid    The Staff ID, if the ticket post is to be created as a staff
    dateline
    fullname
    email
    emailto
    ipaddress
    hasattachments
    creator
    isthirdparty
    ishtml
    isemailed
    issurveycomment
    '''

    __parameters__ = [
        'id',
        'ticketid',
        'subject',
        'contents',
        'userid',
        'staffid',
        'dateline',
        'fullname',
        'email',
        'emailto',
        'ipaddress',
        'hasattachments',
        'creator',
        'isthirdparty',
        'ishtml',
        'isemailed',
        'issurveycomment',
    ]

    __required_add_parameters__ = ['ticketid', 'subject', 'contents']
    __add_parameters__ = ['ticketid', 'subject', 'contents', 'userid', 'staffid']

    __required_save_parameters__ = []
    ''' Save not available for TicketNote. '''
    __save_parameters__ = []
    ''' Save not available for TicketNote. '''

    controller = '/Tickets/TicketPost'

    @classmethod
    def _parse_ticket_post(cls, ticket_post_tree, ticket_id):

        params = dict(
            id=cls._get_int(ticket_post_tree.find('ticketpostid')),
            ticketid=ticket_id,
            subject=cls._get_string(ticket_post_tree.find('subject')),
            contents=cls._get_string(ticket_post_tree.find('contents')),
            userid=cls._get_int(ticket_post_tree.find('userid')),
            staffid=cls._get_int(ticket_post_tree.find('staffid')),
            dateline=cls._get_date(ticket_post_tree.find('dateline')),
            fullname=cls._get_string(ticket_post_tree.find('fullname')),
            email=cls._get_string(ticket_post_tree.find('email')),
            emailto=cls._get_string(ticket_post_tree.find('emailto')),
            ipaddress=cls._get_string(ticket_post_tree.find('ipaddress')),
            hasattachments=cls._get_boolean(ticket_post_tree.find('hasattachments')),
            creator=cls._get_int(ticket_post_tree.find('creator')),
            isthirdparty=cls._get_boolean(ticket_post_tree.find('isthirdparty')),
            ishtml=cls._get_boolean(ticket_post_tree.find('ishtml')),
            isemailed=cls._get_boolean(ticket_post_tree.find('isemailed')),
            issurveycomment=cls._get_boolean(ticket_post_tree.find('issurveycomment')),
        )
        return params

    @classmethod
    def get_all(cls, api, ticketid):
        '''
        Get all of the TicketPosts for a ticket.
        Required:
            ticketid     The unique numeric identifier of the ticket. 
        '''
        response = api._request('%s/ListAll/%s' % (cls.controller, ticketid), 'GET')
        tree = etree.parse(response)
        return [TicketPost(api, **cls._parse_ticket_post(ticket_post_tree, ticketid)) for ticket_post_tree in tree.findall('post')]

    @classmethod
    def get(cls, api, ticketid, id):
        try:
            response = api._request('%s/%s/%s/' % (cls.controller, ticketid, id), 'GET')
        except KayakoResponseError, error:
            if len(error.args) and isinstance(error.args[0], (URLError, HTTPError)) and error.args[0].getcode() == 404:
                return None
        tree = etree.parse(response)
        node = tree.find('post')
        if node is None:
            return None
        params = cls._parse_ticket_post(node, ticketid)
        return TicketPost(api, **params)

    def add(self):
        '''
        Add this TicketPost.
        
        Requires:
            ticketid  The unique numeric identifier of the ticket.
            subject   The ticket post subject
            contents  The ticket post contents
        Requires one of:
            userid    The User ID, if the ticket post is to be created as a user.
            staffid   The Staff ID, if the ticket post is to be created as a staff 
        '''
        if self.id is not UnsetParameter:
            raise KayakoRequestError('Cannot add a pre-existing %s. Use save instead. (id: %s)' % (self.__class__.__name__, self.id))

        parameters = self.add_parameters

        for required_parameter in self.__required_add_parameters__:
            if required_parameter not in parameters:
                raise KayakoRequestError('Cannot add %s: Missing required field: %s.' % (self.__class__.__name__, required_parameter))

        if ('userid' not in parameters and 'staffid' not in parameters) or ('userid' in parameters and 'staffid' in parameters):
            raise KayakoRequestError('To add a TicketPost, just one of the following parameters must be set: userid, staffid. (id: %s)' % self.id)

        response = self.api._request(self.controller, 'POST', **parameters)
        tree = etree.parse(response)
        self.id = int(tree.find('post').find('ticketpostid').text)

    def delete(self):
        if self.ticketid is None or self.ticketid is UnsetParameter:
            raise KayakoRequestError('Cannot delete a TicketPost without being attached to a ticket. The ID of the Ticket (ticketid) has not been specified.')
        self._delete('%s/%s/%s/' % (self.controller, self.ticketid, self.id))

    def __str__(self):
        return '<TicketPost (%s): %s>' % (self.id, self.subject)

class TicketPriority(KayakoObject):
    '''
    Kayako TicketPriority API Object.
    
    id
    title
    displayorder
    frcolorcode
    bgcolorcode
    displayicon
    type
    uservisibilitycustom
    usergroupid

    '''

    controller = '/Tickets/TicketPriority'

    __parameters__ = [
        'id',
        'title',
        'displayorder',
        'frcolorcode',
        'bgcolorcode',
        'displayicon',
        'type',
        'uservisibilitycustom',
        'usergroupid',
    ]

    __required_add_parameters__ = []
    ''' Add not available for TicketPriority. '''
    __add_parameters__ = []
    ''' Add not available for TicketPriority. '''

    __required_save_parameters__ = []
    ''' Save not available for TicketPriority. '''
    __save_parameters__ = []
    ''' Save not available for TicketPriority. '''

    @classmethod
    def _parse_ticket_priority(cls, ticket_priority_tree):

        params = dict(
            id=cls._get_int(ticket_priority_tree.find('id')),
            title=cls._get_string(ticket_priority_tree.find('title')),
            displayorder=cls._get_int(ticket_priority_tree.find('displayorder')),
            frcolorcode=cls._get_string(ticket_priority_tree.find('frcolorcode')),
            bgcolorcode=cls._get_string(ticket_priority_tree.find('bgcolorcode')),
            displayicon=cls._get_string(ticket_priority_tree.find('displayicon')),
            type=cls._get_string(ticket_priority_tree.find('type')),
            uservisibilitycustom=cls._get_boolean(ticket_priority_tree.find('uservisibilitycustom')),
            usergroupid=cls._get_int(ticket_priority_tree.find('usergroupid'), required=False),
        )
        return params

    @classmethod
    def get_all(cls, api):
        response = api._request(cls.controller, 'GET')
        tree = etree.parse(response)
        return [TicketPriority(api, **cls._parse_ticket_priority(ticket_priority_tree)) for ticket_priority_tree in tree.findall('ticketpriority')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('ticketpriority')
        if node is None:
            return None
        params = cls._parse_ticket_priority(tree.find('ticketpriority'))
        return TicketPriority(api, **params)

    def __str__(self):
        return '<TicketPriority (%s): %s>' % (self.id, self.title)

class TicketStatus(KayakoObject):
    '''
    Kayako TicketStatus API Object.
    
    id
    title
    displayorder
    departmentid
    displayicon
    type
    displayinmainlist
    markasresolved
    displaycount
    statuscolor
    statusbgcolor
    resetduetime
    triggersurvey
    staffvisibilitycustom

    '''

    controller = '/Tickets/TicketStatus'

    __parameters__ = [
        'id',
        'title',
        'displayorder',
        'departmentid',
        'displayicon',
        'type',
        'displayinmainlist',
        'markasresolved',
        'displaycount',
        'statuscolor',
        'statusbgcolor',
        'resetduetime',
        'triggersurvey',
        'staffvisibilitycustom',
    ]

    __required_add_parameters__ = []
    ''' Add not available for TicketStatus. '''
    __add_parameters__ = []
    ''' Add not available for TicketStatus. '''

    __required_save_parameters__ = []
    ''' Save not available for TicketStatus. '''
    __save_parameters__ = []
    ''' Save not available for TicketStatus. '''

    @classmethod
    def _parse_ticket_status(cls, ticket_status_tree):

        params = dict(
            id=cls._get_int(ticket_status_tree.find('id')),
            title=cls._get_string(ticket_status_tree.find('title')),
            displayorder=cls._get_int(ticket_status_tree.find('displayorder')),
            departmentid=cls._get_int(ticket_status_tree.find('departmentid')),
            displayicon=cls._get_string(ticket_status_tree.find('displayicon')),
            type=cls._get_string(ticket_status_tree.find('type')),
            displayinmainlist=cls._get_boolean(ticket_status_tree.find('displayinmainlist')),
            markasresolved=cls._get_boolean(ticket_status_tree.find('markasresolved')),
            displaycount=cls._get_int(ticket_status_tree.find('displaycount')),
            statuscolor=cls._get_string(ticket_status_tree.find('statuscolor')),
            statusbgcolor=cls._get_string(ticket_status_tree.find('statusbgcolor')),
            resetduetime=cls._get_boolean(ticket_status_tree.find('resetduetime')),
            triggersurvey=cls._get_boolean(ticket_status_tree.find('triggersurvey')),
            staffvisibilitycustom=cls._get_boolean(ticket_status_tree.find('staffvisibilitycustom')),
        )
        return params

    @classmethod
    def get_all(cls, api):
        response = api._request(cls.controller, 'GET')
        tree = etree.parse(response)
        return [TicketStatus(api, **cls._parse_ticket_status(ticket_status_tree)) for ticket_status_tree in tree.findall('ticketstatus')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('ticketstatus')
        if node is None:
            return None
        params = cls._parse_ticket_status(node)
        return TicketStatus(api, **params)

    def __str__(self):
        return '<TicketStatus (%s): %s>' % (self.id, self.title)

class TicketType(KayakoObject):
    '''
    Kayako TicketType API Object.
    
    id
    title
    displayorder
    departmentid
    displayicon
    type
    uservisibilitycustom

    '''

    controller = '/Tickets/TicketType'

    __parameters__ = [
        'id',
        'title',
        'displayorder',
        'departmentid',
        'displayicon',
        'type',
        'uservisibilitycustom',
    ]

    __required_add_parameters__ = []
    ''' Add not available for TicketType. '''
    __add_parameters__ = []
    ''' Add not available for TicketType. '''

    __required_save_parameters__ = []
    ''' Save not available for TicketType. '''
    __save_parameters__ = []
    ''' Save not available for TicketType. '''

    @classmethod
    def _parse_ticket_type(cls, ticket_type_tree):

        params = dict(
            id=cls._get_int(ticket_type_tree.find('id')),
            title=cls._get_string(ticket_type_tree.find('title')),
            displayorder=cls._get_int(ticket_type_tree.find('displayorder')),
            departmentid=cls._get_int(ticket_type_tree.find('departmentid')),
            displayicon=cls._get_string(ticket_type_tree.find('displayicon')),
            type=cls._get_string(ticket_type_tree.find('type')),
            uservisibilitycustom=cls._get_boolean(ticket_type_tree.find('uservisibilitycustom')),
        )
        return params

    @classmethod
    def get_all(cls, api):
        response = api._request(cls.controller, 'GET')
        tree = etree.parse(response)
        return [TicketType(api, **cls._parse_ticket_type(ticket_type_tree)) for ticket_type_tree in tree.findall('tickettype')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('tickettype')
        if node is None:
            return None
        params = cls._parse_ticket_type(node)
        return TicketType(api, **params)

    def __str__(self):
        return '<TicketType (%s): %s>' % (self.id, self.title)




# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 9, 2011

@author: evan
'''

from lxml import etree

from kayako.core.object import KayakoObject

__all__ = [
    'Staff',
    'StaffGroup',
]

class Staff(KayakoObject):
    '''
    Kayako Staff API Object.
    
    firstname      The first name
    lastname       The last name
    username       The login username
    password       Will only be updated if its specified with the request
    staffgroupid   The staff group ID
    email          The staff email address
    designation    The Designation/Title of Staff
    mobilenumber   The mobile number of Staff user
    signature      The Signature to append to each reply made by staff
    isenabled      Toggle the enabled/disabled property using this flag
    greeting       The default greeting message when the staff accepts a live chat request
    timezone       The default time zone for Staff
    enabledst      Toggle the enabling/disabling of automatic DST calculation 
    '''

    controller = '/Base/Staff'

    __parameters__ = [
        'id',
        'firstname',
        'lastname',
        'username',
        'password',
        'staffgroupid',
        'email',
        'designation',
        'mobilenumber',
        'signature',
        'isenabled',
        'greeting',
        'timezone',
        'enabledst',
    ]

    __required_add_parameters__ = ['firstname', 'lastname', 'username', 'email', 'password', 'staffgroupid']
    __add_parameters__ = ['firstname', 'lastname', 'username', 'email', 'password', 'staffgroupid', 'designation', 'mobilenumber', 'signature', 'isenabled', 'greeting', 'timezone', 'enabledst']

    __required_save_parameters__ = ['firstname', 'lastname']
    __save_parameters__ = ['firstname', 'lastname', 'username', 'email', 'password', 'staffgroupid', 'designation', 'mobilenumber', 'signature', 'isenabled', 'greeting', 'timezone', 'enabledst']

    @classmethod
    def _parse_staff(cls, staff_tree):
        params = dict(
            id=cls._get_int(staff_tree.find('id')),
            firstname=cls._get_string(staff_tree.find('firstname')),
            lastname=cls._get_string(staff_tree.find('lastname')),
            username=cls._get_string(staff_tree.find('username')),
            #password is never present in the response
            staffgroupid=cls._get_int(staff_tree.find('staffgroupid')),
            email=cls._get_string(staff_tree.find('email')),
            designation=cls._get_string(staff_tree.find('designation')),
            mobilenumber=cls._get_string(staff_tree.find('mobilenumber')),
            signature=cls._get_string(staff_tree.find('signature')),
            isenabled=cls._get_boolean(staff_tree.find('isenabled')),
            greeting=cls._get_string(staff_tree.find('greeting')),
            timezone=cls._get_string(staff_tree.find('timezone')),
            enabledst=cls._get_boolean(staff_tree.find('enabledst')),
        )
        return params

    @classmethod
    def get_all(cls, api):
        response = api._request(cls.controller, 'GET')
        tree = etree.parse(response)
        return [Staff(api, **cls._parse_staff(staff_tree)) for staff_tree in tree.findall('staff')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('staff')
        if node is None:
            return None
        params = cls._parse_staff(node)
        return Staff(api, **params)

    def add(self):
        response = self._add(self.controller)
        tree = etree.parse(response)
        self.id = int(tree.find('staff').find('id').text)

    def save(self):
        self._save('%s/%s/' % (self.controller, self.id))

    def delete(self):
        self._delete('%s/%s/' % (self.controller, self.id))

    def __repr__(self):
        return '<Staff (%s): %s %s (%s)>' % (self.id, self.firstname, self.lastname, self.username)

class StaffGroup(KayakoObject):
    '''
    Kayako StaffGroup API object.
    
    $id$     The numeric identifier of the staff group.
    title    The title of the staff group.
    isadmin  1 or 0, boolean controlling whether or not staff members assigned to this group are Administrators.
    '''

    controller = '/Base/StaffGroup'


    __parameters__ = ['id', 'title', 'isadmin']

    __required_add_parameters__ = ['title', 'isadmin']
    __add_parameters__ = ['title', 'isadmin']

    __required_save_parameters__ = ['title']
    __save_parameters__ = ['title', 'isadmin']


    @classmethod
    def _parse_staff_group(cls, staff_group_tree):
        params = dict(
            id=cls._get_int(staff_group_tree.find('id')),
            title=cls._get_string(staff_group_tree.find('title')),
            isadmin=cls._get_boolean(staff_group_tree.find('isadmin')),
        )
        return params

    @classmethod
    def get_all(cls, api):
        response = api._request(cls.controller, 'GET')
        tree = etree.parse(response)
        return [StaffGroup(api, **cls._parse_staff_group(staff_group_tree)) for staff_group_tree in tree.findall('staffgroup')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('staffgroup')
        if node is None:
            return None
        params = cls._parse_staff_group(node)
        return StaffGroup(api, **params)

    def add(self):
        response = self._add(self.controller)
        tree = etree.parse(response)
        self.id = int(tree.find('staffgroup').find('id').text)

    def save(self):
        self._save('%s/%s/' % (self.controller, self.id))

    def delete(self):
        self._delete('%s/%s/' % (self.controller, self.id))

    def __str__(self):
        return '<StaffGroup (%s): %s>' % (self.id, self.title)









# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 5, 2011

@author: evan
'''

import base64
import hashlib
import hmac
import random
import urllib
import urllib2
import time
from datetime import datetime

from kayako.exception import KayakoRequestError, KayakoResponseError, KayakoInitializationError
from kayako.core.lib import FOREVER

class KayakoAPI:
    ''' 
    API Wrapper for Kayako. 
    -----------------------
    
    **Usage:**

    ::
    
        from kayako import KayakoAPI, User, Ticket, Department, UnsetParameter
        api = KayakoAPI('http://kayako.foo.com/api/index.php', 's8v092-2lksd-9cso-c2', 'somesecret')
        departments = api.get_all(Department)
        for department in departments:
            # Print every ticket in every department
            tickets = api.get_all(Ticket, department.id)
            for ticket in tickets:
                print department, ticket
        >>> <Department...> <Ticket...>
        >>> <Department...> <Ticket...>
        >>> <Department...> <Ticket...>
        
    **Add an object**
    
    ::
    
        department = api.create(Department)
        department.title = 'Food Department' # Department author was hungry
        department.module = 'tickets'
        department.type = 'private'
        assert department.id is UnsetParameter
        department.add()
        assert department.id is not UnsetParameter
        department.title = 'Foo Department' # 'Food' was supposed to be 'Foo'
        department.save()
        department.delete()
    
    **API Factory Methods:**
    
    ``api.create(Object, *args, **kwargs)``
    
        Create a new KayakoObject of the type given, passing in args and kwargs.
        
    ``api.get_all(Object, *args, **kwargs)``
    
        *Get all Kayako Objects of the given type.*
        *In most cases, all items are returned.*
        
        e.x.::
            api.get_all(Department)
            >>> [<Department....>, ....]
    
        *Special Cases:*
        
            ``api.get_all(User, marker=1, maxitems=1000)``
                Return all Users from userid ``marker`` with up to ``maxitems`` 
                results (max 1000.)
                
            ``api.get_all(Ticket, departmentid, ticketstatusid=-1, ownerstaffid=-1, userid=-1)``
                Return all Tickets filtered by the required argument 
                ``departmentid`` and by the optional keyword arguments.
                
            ``api.get_all(TicketAttachment, ticketid)``
                Return all TicketAttachments for a Ticket with the given ID.
                
            ``api.get_all(TicketPost, ticketid)``
                Return all TicketPosts for a Ticket with the given ID.
    
    ``api.get(Object, *args)``
    
        *Get a Kayako Object of the given type by ID.*
        
        e.x.
            ``api.get(User, 112359)``
            ``>>> <User (112359)....>``
        
        *Special Cases:*
            
            ``api.get(TicketAttachment, ticketid, attachmentid)``
                Return a ``TicketAttachment`` for a ``Ticket`` with the given Ticket
                ID and TicketAttachment ID.  Getting a specific ``TicketAttachment``
                gets a ``TicketAttachment`` with the actual attachment contents.
            
            ``api.get(TicketPost, ticketid, ticketpostid)``
                Return a ``TicketPost`` for a ticket with the given Ticket ID and
                TicketPost ID.
                
    **Object persistence methods**
    
    ``kayakoobject.add()``
        *Adds the instance to Kayako.*
    ``kayakoobject.save()``
        *Saves an existing object the instance to Kayako.*
    ``kayakoobject.delete()``
        *Removes the instance from Kayako*
        
    These methods can raise exceptions:
    
        Raises ``KayakoRequestError`` if one of the following is true:
            - The action is not available for the object
            - A required object parameter is UnsetParameter or None (add/save)
            - The API URL cannot be reached
            
        Raises ``KayakoResponseError`` if one of the following is true:
            - There is an error with the request (not HTTP 200 Ok)
            - The XML is in an unexpected format indicating a possible Kayako version mismatch (expects 4.01.204)
        
    **Quick Reference**
    
    ================= ====================================================================== ========================= ======= ======= =====================
    Object            Get All                                                                Get                       Add     Save    Delete
    ================= ====================================================================== ========================= ======= ======= =====================
    Department        Yes                                                                    Yes                       Yes     Yes     Yes
    Staff             Yes                                                                    Yes                       Yes     Yes     Yes
    StaffGroup        Yes                                                                    Yes                       Yes     Yes     Yes
    Ticket            departmentid, ticketstatusid= -1, ownerstaffid= -1, userid= -1         Yes                       Yes     Yes     Yes
    TicketAttachment  ticketid                                                               ticketid, attachmentid    Yes     No      Yes
    TicketNote        ticketid                                                               No                        Yes     No      No (delete ticket)
    TicketPost        ticketid                                                               ticketid, postid          Yes     No      Yes
    TicketPriority    Yes                                                                    Yes                       No      No      No
    TicketStatus      Yes                                                                    Yes                       No      No      No
    TicketType        Yes                                                                    Yes                       No      No      No
    User              marker=1, maxitems=1000                                                Yes                       Yes     Yes     Yes
    UserGroup         Yes                                                                    Yes                       Yes     Yes     Yes
    UserOrganization  Yes                                                                    Yes                       Yes     Yes     Yes
    ================= ====================================================================== ========================= ======= ======= =====================
    
    '''

    def __init__(self, api_url, api_key, secret_key):
        ''' 
        Creates a new wrapper that will make requests to the given URL using
        the authentication provided.
        '''

        if not api_url:
            raise KayakoInitializationError('API URL not specified.')
        self.api_url = api_url

        if not api_key:
            raise KayakoInitializationError('API Key not specified.')
        self.secret_key = secret_key

        if not secret_key:
            raise KayakoInitializationError('Secret Key not specified.')
        self.api_key = api_key

    ## { Communication Layer

    def _sanitize_parameter(self, parameter):
        '''
        Sanitize a specific object.
        
        - Convert None types to empty strings
        - Convert FOREVER to '0'
        - Convert lists/tuples into sanitized lists
        - Convert objects to strings
        '''

        if parameter is None:
            return ''
        elif parameter is FOREVER:
            return '0'
        elif parameter is True:
            return '1'
        elif parameter is False:
            return '0'
        elif isinstance(parameter, datetime):
            return str(int(time.mktime(parameter.timetuple())))
        elif isinstance(parameter, (list, tuple, set)):
            return [self._sanitize_parameter(item) for item in parameter if item not in ['', None]]
        else:
            return str(parameter)

    def _sanitize_parameters(self, **parameters):
        '''
        Sanitize a dictionary of parameters for a request.
        '''
        result = dict()
        for key, value in parameters.iteritems():
            result[key] = self._sanitize_parameter(value)
        return result

    def _post_data(self, **parameters):
        '''
        Turns parameters into application/x-www-form-urlencoded format.
        '''
        data = None
        first = True
        for key, value in parameters.iteritems():
            if isinstance(value, list):
                if len(value):
                    for sub_value in value:
                        if first:
                            data = '%s[]=%s' % (key, urllib2.quote(sub_value))
                            first = False
                        else:
                            data = '%s&%s[]=%s' % (data, key, urllib2.quote(sub_value))
                else:
                    if first:
                        data = '%s[]=' % key
                        first = False
                    else:
                        data = '%s&%s[]=' % (data, key)
            elif first:
                data = '%s=%s' % (key, urllib2.quote(value))
                first = False
            else:
                data = '%s&%s=%s' % (data, key, urllib2.quote(value))
        return data

    def _generate_signature(self):
        '''
        Generates random salt and an encoded signature using SHA256.
        '''
        # Generate random 10 digit number
        salt = str(random.getrandbits(32))
        # Use HMAC to encrypt the secret key using the salt with SHA256
        encrypted_signature = hmac.new(self.secret_key, msg=salt, digestmod=hashlib.sha256).digest()
        # Encode the bytes into base 64
        b64_encoded_signature = base64.b64encode(encrypted_signature)
        return salt, b64_encoded_signature

    def _request(self, controller, method, **parameters):
        '''
        Get a response from the specified controller using the given parameters.
        '''
        salt, b64signature = self._generate_signature()

        if method == 'GET':
            url = '%s?e=%s&apikey=%s&salt=%s&signature=%s' % (self.api_url, urllib.quote(controller), urllib.quote(self.api_key), salt, urllib.quote(b64signature))
            # Append additional query args if necessary
            data = self._post_data(**self._sanitize_parameters(**parameters)) if parameters else None
            if data:
                url = '%s&%s' % (url, data)
            request = urllib2.Request(url)
        elif method == 'POST' or method == 'PUT':
            url = '%s?e=%s' % (self.api_url, urllib.quote(controller))
            # Auth parameters go in the body for these methods
            parameters['apikey'] = self.api_key
            parameters['salt'] = salt
            parameters['signature'] = b64signature
            data = self._post_data(**self._sanitize_parameters(**parameters))
            request = urllib2.Request(url, data=data, headers={'Content-length' : len(data) if data else 0})
            request.get_method = lambda: method
        elif method == 'DELETE': # DELETE
            url = '%s?e=%s&apikey=%s&salt=%s&signature=%s' % (self.api_url, urllib.quote(controller), urllib.quote(self.api_key), salt, urllib.quote(b64signature))
            data = self._post_data(**self._sanitize_parameters(**parameters))
            request = urllib2.Request(url, data=data, headers={'Content-length' : len(data) if data else 0})
            request.get_method = lambda: method
        else:
            raise KayakoRequestError('Invalid request method: %s not supported.' % method)

#        print 'REQUEST:'
#        print method
#        print 'URL:'
#        print url.replace('&', '\n&') if url else 'None'
#        print 'DATA:'
#        print data.replace('&', '\n&') if data else 'None'
#        print '------'

        try:
            response = urllib2.urlopen(request)
        except urllib2.HTTPError, error:
#            print error.read()
            raise KayakoResponseError(error)
        except urllib2.URLError, error:
#            print error.read()
            raise KayakoRequestError(error)
        return response

    ## { Persistence Layer

    def create(self, object, *args, **kwargs):
        '''
        Create a new KayakoObject of the type given, passing in args and kwargs.
        '''
        return object(self, *args, **kwargs)

    def get_all(self, object, *args, **kwargs):
        '''
        Get all Kayako Objects of the given type.
        By default, all items are returned. 
        
        e.x.
            api.get_all(Department)
            >>> [<Department....>, ....]
    
        Special Cases:
        
            api.get_all(User, marker=1, maxitems=1000)
                Return all Users from userid ``marker`` with up to ``maxitems``
                results (max 1000.)
                
            api.get_all(Ticket, departmentid, ticketstatusid= -1, 
              ownerstaffid= -1, userid= -1)
                Return all Tickets filtered by the required argument
                ``departmentid`` and by the optional keyword arguments.
                
            api.get_all(TicketAttachment, ticketid)
                Return all TicketAttachments for a Ticket with the given ID.
                
            api.get_all(TicketPost, ticketid)
                Return all TicketPosts for a Ticket with the given ID.
                
        '''
        return object.get_all(self, *args, **kwargs)

    def get(self, object, *args):
        '''
        Get a Kayako Object of the given type by ID.
        
        e.x.
            api.get(User, 112359)
            >>> <User....>
        
        Special Cases:
            
            api.get(TicketAttachment, ticketid, attachmentid)
                Return a TicketAttachment for a ticket with the given Ticket ID
                and TicketAttachment ID.  Getting a specific TicketAttachment
                gets a TicketAttachment with the actual attachment contents.
            
            api.get(TicketPost, ticketid, ticketpostid)
                Return a TicketPost for a ticket with the given Ticket ID and
                TicketPost ID.
        
        '''


        return object.get(self, *args)

    def __str__(self):
        return '<KayakoAPI: %s>' % self.api_url

    def __repr__(self):
        return 'KayakoAPI(%s, "some_key", "some_secret")' % (self.api_url)



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 9, 2011

@author: evan
'''

from lxml import etree

from kayako.core.object import KayakoObject

__all__ = [
    'User',
    'UserGroup',
    'UserOrganization',
]

class User(KayakoObject):
    '''
    Kayako User API Object.
    
    fullname              The full name of User.
    usergroupid           The User Group ID to assign to this User.
    email                 The Email Address of User, you can send multiple email addresses by using "email[]" as the key
    userorganizationid    The User Organization ID
    salutation            The User Salutation, available options are: Mr., Ms., Mrs., Dr.
    designation           The User Designation/Title
    phone                 The Phone Number
    isenabled             Toggle whether the user is to be enabled/disabled
    userrole              The User Role, available options are: user, manager. Default: user
    timezone              The Time Zone
    enabledst             Enable Daylight Savings?
    slaplanid             The SLA Plan ID to assign to the user
    slaplanexpiry         The SLA Plan Expiry, 0 = never expires
    userexpiry            The User Expiry, 0 = never expires 
    dateline             
    lastvisit
    '''

    controller = '/Base/User'

    __parameters__ = [
        'id',
        'fullname',
        'usergroupid',
        'email',
        'password',
        'userorganizationid',
        'salutation',
        'designation',
        'phone',
        'isenabled',
        'userrole',
        'timezone',
        'enabledst',
        'slaplanid',
        'slaplanexpiry',
        'userexpiry',
        'dateline',
        'lastvisit',
        'sendwelcomeemail',
    ]

    __required_add_parameters__ = ['fullname', 'usergroupid', 'password', 'email']
    __add_parameters__ = ['fullname', 'usergroupid', 'password', 'email', 'userorganizationid', 'salutation', 'designation', 'phone', 'isenabled', 'userrole', 'timezone', 'enabledst', 'slaplanid', 'slaplanexpiry', 'userexpiry', 'sendwelcomeemail']

    __required_save_parameters__ = ['fullname']
    __save_parameters__ = ['fullname', 'usergroupid', 'email', 'userorganizationid', 'salutation', 'designation', 'phone', 'isenabled', 'userrole', 'timezone', 'enabledst', 'slaplanid', 'slaplanexpiry', 'userexpiry']

    @classmethod
    def _parse_user(cls, user_tree):
        params = dict(
            id=cls._get_int(user_tree.find('id')),
            fullname=cls._get_string(user_tree.find('fullname')),
            usergroupid=cls._get_int(user_tree.find('usergroupid')),
            email=[cls._get_string(email_tree.find('email')) for email_tree in user_tree.findall('email')],
            userorganizationid=cls._get_int(user_tree.find('userorganizationid'), required=False),
            salutation=cls._get_string(user_tree.find('salutation')),
            designation=cls._get_string(user_tree.find('designation')),
            phone=cls._get_string(user_tree.find('phone')),
            isenabled=cls._get_boolean(user_tree.find('isenabled')),
            userrole=cls._get_string(user_tree.find('userrole')),
            timezone=cls._get_string(user_tree.find('timezone')),
            enabledst=cls._get_boolean(user_tree.find('enabledst')),
            slaplanid=cls._get_int(user_tree.find('slaplanid')),
            slaplanexpiry=cls._get_date(user_tree.find('slaplanexpiry')),
            userexpiry=cls._get_date(user_tree.find('userexpiry')),
            dateline=cls._get_date(user_tree.find('dateline')),
            lastvisit=cls._get_date(user_tree.find('lastvisit')),
        )
        return params

    @classmethod
    def get_all(cls, api, marker=1, maxitems=1000):
        '''
        Returns the users starting at User ID ``marker`` pulling in a maximum
        ``maxitems`` number of Users.
        '''
        response = api._request('%s/%s/%s/' % (cls.controller, marker, maxitems), 'GET')
        tree = etree.parse(response)
        return [User(api, **cls._parse_user(user_tree)) for user_tree in tree.findall('user')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('user')
        if node is None:
            return None
        params = cls._parse_user(node)
        return User(api, **params)

    def add(self):
        response = self._add(self.controller)
        tree = etree.parse(response)
        self.id = int(tree.find('user').find('id').text)

    def save(self):
        self._save('%s/%s/' % (self.controller, self.id))

    def delete(self):
        self._delete('%s/%s/' % (self.controller, self.id))

    def __str__(self):
        return '<User (%s): %s %s>' % (self.id, self.fullname, self.email)

class UserGroup(KayakoObject):
    '''
    Kayako UserGroup API Object.
    
    title      The title of the user group.
    grouptype  The type of user group ('guest' or 'registered'). 
    ismaster
    '''

    controller = '/Base/UserGroup'

    __parameters__ = [
        'id',
        'title',
        'grouptype',
        'ismaster',
    ]

    __required_add_parameters__ = ['title', 'grouptype']
    __add_parameters__ = ['title', 'grouptype']

    __required_save_parameters__ = ['title']
    __save_parameters__ = ['title']

    @classmethod
    def _parse_user_group(cls, user_group_tree):
        params = dict(
            id=cls._get_int(user_group_tree.find('id')),
            title=cls._get_string(user_group_tree.find('title')),
            grouptype=cls._get_string(user_group_tree.find('grouptype')),
            ismaster=cls._get_boolean(user_group_tree.find('ismaster')),
        )
        return params

    @classmethod
    def get_all(cls, api):
        response = api._request(cls.controller, 'GET')
        tree = etree.parse(response)
        return [UserGroup(api, **cls._parse_user_group(user_group_tree)) for user_group_tree in tree.findall('usergroup')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('usergroup')
        if node is None:
            return None
        params = cls._parse_user_group(node)
        return UserGroup(api, **params)

    def add(self):
        response = self._add(self.controller)
        tree = etree.parse(response)
        self.id = int(tree.find('usergroup').find('id').text)

    def save(self):
        self._save('%s/%s/' % (self.controller, self.id))

    def delete(self):
        self._delete('%s/%s/' % (self.controller, self.id))

    def __str__(self):
        return '<UserGroup (%s): %s (%s)>' % (self.id, self.title, self.grouptype)

class UserOrganization(KayakoObject):
    '''
    Kayako UserOrganization API Object.
    
    name               The name of the user organization.
    organizationtype   The type of user organization ('restricted' or 'shared').
    address      
    city      
    state      
    postalcode      
    country      
    phone      
    fax      
    website      
    slaplanid          The SLA Plan ID to link with this organization
    slaplanexpiry      The UNIX timestamp by which to ignore the SLA Plan, 0 = never expires
    '''

    controller = '/Base/UserOrganization'

    __parameters__ = [
        'id',
        'name',
        'organizationtype',
        'address',
        'city',
        'state',
        'postalcode',
        'country',
        'phone',
        'fax',
        'website',
        'slaplanid',
        'slaplanexpiry',
        'dateline',
        'lastupdate',
    ]

    __required_add_parameters__ = ['name', 'organizationtype']
    __add_parameters__ = ['name', 'organizationtype', 'address', 'city', 'state', 'postalcode', 'country', 'phone', 'fax', 'website', 'slaplanid', 'slaplanexpiry']

    __required_save_parameters__ = ['name']
    __save_parameters__ = ['name', 'organizationtype', 'address', 'city', 'state', 'postalcode', 'country', 'phone', 'fax', 'website', 'slaplanid', 'slaplanexpiry']

    @classmethod
    def _parse_user_organization(cls, user_organization_tree):
        params = dict(
            id=cls._get_int(user_organization_tree.find('id')),
            name=cls._get_string(user_organization_tree.find('name')),
            organizationtype=cls._get_string(user_organization_tree.find('organizationtype')),
            address=cls._get_string(user_organization_tree.find('address')),
            city=cls._get_string(user_organization_tree.find('city')),
            state=cls._get_string(user_organization_tree.find('state')),
            postalcode=cls._get_string(user_organization_tree.find('postalcode')),
            country=cls._get_string(user_organization_tree.find('country')),
            phone=cls._get_string(user_organization_tree.find('phone')),
            fax=cls._get_string(user_organization_tree.find('fax')),
            website=cls._get_string(user_organization_tree.find('website')),
            dateline=cls._get_date(user_organization_tree.find('dateline')),
            lastupdate=cls._get_date(user_organization_tree.find('lastupdate')),
            slaplanid=cls._get_int(user_organization_tree.find('slaplanid')),
            slaplanexpiry=cls._get_date(user_organization_tree.find('slaplanexpiry')),
        )
        return params

    @classmethod
    def get_all(cls, api):
        response = api._request(cls.controller, 'GET')
        tree = etree.parse(response)
        return [UserOrganization(api, **cls._parse_user_organization(user_organization_tree)) for user_organization_tree in tree.findall('userorganization')]

    @classmethod
    def get(cls, api, id):
        response = api._request('%s/%s/' % (cls.controller, id), 'GET')
        tree = etree.parse(response)
        node = tree.find('userorganization')
        if node is None:
            return node
        params = cls._parse_user_organization(node)
        return UserOrganization(api, **params)

    def add(self):
        response = self._add(self.controller)
        tree = etree.parse(response)
        self.id = int(tree.find('userorganization').find('id').text)

    def save(self):
        self._save('%s/%s/' % (self.controller, self.id))

    def delete(self):
        self._delete('%s/%s/' % (self.controller, self.id))

    def __str__(self):
        return '<UserOrganization (%s): %s>' % (self.id, self.name)









# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 9, 2011

@author: evan
'''
from kayako.test import KayakoAPITest

class TestTicketPriority(KayakoAPITest):

    def test_get_all(self):
        from kayako.objects import TicketPriority
        result = self.api.get_all(TicketPriority)
        assert isinstance(result, list)

    def test_get(self):
        from kayako.objects import TicketPriority
        result = self.api.get(TicketPriority, 1)
        assert 'TicketPriority' in str(result)
        self.assertEqual(result.id, 1)

    def test_get_nonexistant(self):
        from kayako.objects import TicketPriority
        self.assertEqual(self.api.get(TicketPriority, '123123'), None)

class TestTicketStatus(KayakoAPITest):

    def test_get_all(self):
        from kayako.objects import TicketStatus
        result = self.api.get_all(TicketStatus)
        assert isinstance(result, list)

    def test_get(self):
        from kayako.objects import TicketStatus
        result = self.api.get(TicketStatus, 1)
        assert 'TicketStatus' in str(result)
        self.assertEqual(result.id, 1)

    def test_get_nonexistant(self):
        from kayako.objects import TicketStatus
        self.assertEqual(self.api.get(TicketStatus, '123123'), None)

class TestTicketType(KayakoAPITest):

    def test_get_all(self):
        from kayako.objects import TicketType
        result = self.api.get_all(TicketType)
        #print result
        assert isinstance(result, list)

    def test_get(self):
        from kayako.objects import TicketType
        result = self.api.get(TicketType, 1)
        assert 'TicketType' in str(result)
        self.assertEqual(result.id, 1)

    def test_get_nonexistant(self):
        from kayako.objects import TicketType
        self.assertEqual(self.api.get(TicketType, '123123'), None)






# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 10, 2011

@author: evan
'''

from kayako.test import KayakoAPITest

class TestTicketPost(KayakoAPITest):

    def test_get_nonexistant(self):
        from kayako.objects import Department, Ticket, TicketPost

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        obj = api.get(TicketPost, ticket.id, 'abc123')

        ticket.delete()

        assert obj is None

    def test_add_get_userid(self):
        from kayako.objects import Department, Ticket, TicketPost
        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'test_post'
        ticket_post.contents = 'testing a post'
        ticket_post.userid = 1
        ticket_post.add()

        obj2 = api.get(TicketPost, ticket.id, ticket_post.id)

        ticket.delete()
        assert obj2 is not None

    def test_add_get_staffid(self):
        from kayako.objects import Department, Ticket, TicketPost
        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'test_post'
        ticket_post.contents = 'testing a post'
        ticket_post.staffid = 1
        ticket_post.add()

        obj2 = api.get(TicketPost, ticket.id, ticket_post.id)

        ticket.delete()
        assert obj2 is not None


    def test_get_all(self):
        from kayako.objects import Department, Ticket, TicketPost

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'test_post'
        ticket_post.contents = 'testing a post'
        ticket_post.userid = 1
        ticket_post.add()

        result = self.api.get_all(TicketPost, ticket.id)
        assert isinstance(result, list)
        assert result

        ticket_post.delete()
        ticket.delete()

    def test_get(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import Department, Ticket, TicketPost

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        assert ticket.id is not UnsetParameter

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'test_post'
        ticket_post.contents = 'testing a post'
        ticket_post.userid = 1
        ticket_post.add()
        assert ticket_post.ticketid == ticket.id

        ticket = api.get(Ticket, ticket.id)
        assert ticket.posts[0].ticketid == ticket.id

        result = self.api.get(TicketPost, ticket.id, ticket_post.id)
        assert result.id == ticket_post.id

        assert 'TicketPost ' in str(result)

        ticket_post.delete()
        ticket.delete()

    def test_add_missing_ticketid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketPost

        ticket_post = self.api.create(TicketPost)

        ticket_post.subject = 'test note'
        ticket_post.contents = 'this is just a test'
        ticket_post.staffid = 1

        self.assertRaises(KayakoRequestError, ticket_post.add)

    def test_add_missing_subject(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketPost

        ticket_post = self.api.create(TicketPost)

        ticket_post.ticketid = 1
        ticket_post.contents = 'this is just a test'
        ticket_post.staffid = 1

        self.assertRaises(KayakoRequestError, ticket_post.add)

    def test_add_missing_contents(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketPost

        ticket_post = self.api.create(TicketPost)

        ticket_post.ticketid = 1
        ticket_post.subject = 'test note'
        ticket_post.staffid = 1

        self.assertRaises(KayakoRequestError, ticket_post.add)

    def test_add_missing_userid_and_staffid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketPost

        ticket_post = self.api.create(TicketPost)

        ticket_post.ticketid = 1
        ticket_post.subject = 'test note'
        ticket_post.contents = 'this is just a test'

        self.assertRaises(KayakoRequestError, ticket_post.add)

    def test_add_with_both_userid_and_staffid(self):
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketPost

        ticket_post = self.api.create(TicketPost)

        ticket_post.ticketid = 1
        ticket_post.subject = 'test note'
        ticket_post.contents = 'this is just a test'
        ticket_post.userid = 'test name'
        ticket_post.staffid = 1

        self.assertRaises(KayakoRequestError, ticket_post.add)

    def test_add_delete(self):
        from kayako.core.lib import UnsetParameter
        from kayako.objects import Department, Ticket, TicketPost

        api = self.api

        depts = api.get_all(Department)
        for dept in depts:
            if dept.module == 'tickets':
                break

        ticket = api.create(Ticket)
        ticket.subject = 'DELETE_ME'
        ticket.fullname = 'Unit Test'
        ticket.email = 'test@example.com'
        ticket.contents = 'test'
        ticket.departmentid = dept.id
        ticket.ticketstatusid = 1
        ticket.ticketpriorityid = 1
        ticket.tickettypeid = 1
        ticket.userid = 1
        ticket.ownerstaffid = 1
        ticket.type = 'default'
        ticket.add()

        ticket_post = api.create(TicketPost)
        ticket_post.ticketid = ticket.id
        ticket_post.subject = 'DELETE_ME'
        ticket_post.contents = 'testing a post'
        ticket_post.userid = 1
        ticket_post.add()

        assert ticket_post.id is not UnsetParameter
        ticket_post.delete()
        ticket.delete()

    def test_delete_unadded(self):
        from kayako.core.lib import UnsetParameter
        from kayako.exception import KayakoRequestError
        from kayako.objects import TicketPost
        ticket_post = self.api.create(TicketPost)
        ticket_post.id = UnsetParameter
        ticket_post.ticketid = 1
        self.assertRaises(KayakoRequestError, ticket_post.delete)
        ticket_post.id = 1
        ticket_post.ticketid = UnsetParameter
        self.assertRaises(KayakoRequestError, ticket_post.delete)



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 5, 2011

@author: evan
'''

import unittest

class TestKayakoObject(unittest.TestCase):

    @property
    def api(self):
        from kayako.api import KayakoAPI
        return KayakoAPI('url', 'key', 'secret')

    @property
    def kayako_object(self):
        from kayako.core.object import KayakoObject
        return KayakoObject(self.api)

    def test_kayako_object(self):
        self.assert_(self.kayako_object)

    def test_kayko_get(self):
        from kayako.exception import KayakoMethodNotImplementedError
        self.assertRaises(KayakoMethodNotImplementedError, self.kayako_object.get, self.api, 123)

    def test_kayko_add(self):
        from kayako.exception import KayakoMethodNotImplementedError
        self.assertRaises(KayakoMethodNotImplementedError, self.kayako_object.add)

    def test_kayko_save(self):
        from kayako.exception import KayakoMethodNotImplementedError
        self.assertRaises(KayakoMethodNotImplementedError, self.kayako_object.save)

    def test_kayko_delete(self):
        from kayako.exception import KayakoMethodNotImplementedError
        self.assertRaises(KayakoMethodNotImplementedError, self.kayako_object.delete)



# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 10, 2011

@author: evan
'''

import unittest

class TestKayakoRequestParser(unittest.TestCase):

    def _etree_with_data(self, data):
        from lxml import etree
        return etree.fromstring('<data>%s</data>' % data)

    def test__parse_int_required(self):
        from kayako.exception import KayakoResponseError
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._parse_int('123', required=True) == 123
        self.assertRaises(KayakoResponseError, KayakoRequestParser._parse_int, None, required=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._parse_int, '', required=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._parse_int, 'abc', required=True)

    def test__parse_int_optional(self):
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._parse_int('123', required=False, strict=False) == 123
        assert KayakoRequestParser._parse_int(None, required=False, strict=False) == None
        assert KayakoRequestParser._parse_int('', required=False, strict=False) == None
        assert KayakoRequestParser._parse_int('abc', required=False, strict=False) == None

    def test__parse_int_strict(self):
        from kayako.exception import KayakoResponseError
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._parse_int('123', required=False, strict=True) == 123
        assert KayakoRequestParser._parse_int(None, required=False, strict=True) == None
        assert KayakoRequestParser._parse_int('', required=False, strict=True) == None
        self.assertRaises(KayakoResponseError, KayakoRequestParser._parse_int, 'abc', required=False, strict=True)

    def test__get_int_required(self):
        from kayako.exception import KayakoResponseError
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._get_int(self._etree_with_data('123'), required=True) == 123
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_int, None, required=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_int, self._etree_with_data(''), required=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_int, self._etree_with_data('abc'), required=True)

    def test__get_int_optional(self):
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._get_int(self._etree_with_data('123'), required=False, strict=False) == 123
        assert KayakoRequestParser._get_int(None, required=False, strict=False) == None
        assert KayakoRequestParser._get_int(self._etree_with_data(''), required=False, strict=False) == None
        assert KayakoRequestParser._get_int(self._etree_with_data('abc'), required=False, strict=False) == None

    def test__get_int_strict(self):
        from kayako.exception import KayakoResponseError
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._get_int(self._etree_with_data('123'), required=False, strict=True) == 123
        assert KayakoRequestParser._get_int(None, required=False, strict=True) == None
        assert KayakoRequestParser._get_int(self._etree_with_data(''), required=False, strict=True) == None
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_int, self._etree_with_data('abc'), required=False, strict=True)

    def test__get_string(self):
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._get_string(self._etree_with_data('123')) == '123'
        assert KayakoRequestParser._get_string(None) == None
        assert KayakoRequestParser._get_string(self._etree_with_data('')) == None

    def test__get_boolean_required(self):
        from kayako.exception import KayakoResponseError
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._get_boolean(self._etree_with_data('1'), required=True) == True
        assert KayakoRequestParser._get_boolean(self._etree_with_data('0'), required=True) == False
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_boolean, None, required=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_boolean, self._etree_with_data(''), required=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_boolean, self._etree_with_data('abc'), required=True)

    def test__get_boolean_optional(self):
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._get_boolean(self._etree_with_data('1'), required=False, strict=False) == True
        assert KayakoRequestParser._get_boolean(self._etree_with_data('0'), required=False, strict=False) == False
        assert KayakoRequestParser._get_boolean(None, required=False, strict=False) == None
        assert KayakoRequestParser._get_boolean(self._etree_with_data(''), required=False, strict=False) == None
        assert KayakoRequestParser._get_boolean(self._etree_with_data('abc'), required=False, strict=False) == None
        assert KayakoRequestParser._get_boolean(self._etree_with_data('2'), required=False, strict=False) == True

    def test__get_boolean_strict(self):
        from kayako.exception import KayakoResponseError
        from kayako.core.object import KayakoRequestParser

        assert KayakoRequestParser._get_boolean(self._etree_with_data('1'), required=False, strict=True) == True
        assert KayakoRequestParser._get_boolean(self._etree_with_data('0'), required=False, strict=True) == False
        assert KayakoRequestParser._get_boolean(None, required=False, strict=True) == None
        assert KayakoRequestParser._get_boolean(self._etree_with_data(''), required=False, strict=True) == None
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_boolean, self._etree_with_data('abc'), required=False, strict=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_boolean, self._etree_with_data('2'), required=False, strict=True)

    def test__get_date_required(self):
        import time
        from datetime import datetime
        from kayako.exception import KayakoResponseError
        from kayako.core.lib import FOREVER
        from kayako.core.object import KayakoRequestParser

        timestamp = int(time.mktime(datetime.now().timetuple()))
        now = datetime.fromtimestamp(timestamp)

        self.assertEqual(KayakoRequestParser._get_date(self._etree_with_data(timestamp), required=True), now)
        assert KayakoRequestParser._get_date(self._etree_with_data('0'), required=True) == FOREVER
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_date, None, required=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_date, self._etree_with_data(''), required=True)
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_date, self._etree_with_data('abc'), required=True)

    def test__get_date_optional(self):
        import time
        from datetime import datetime
        from kayako.core.lib import FOREVER
        from kayako.core.object import KayakoRequestParser

        timestamp = int(time.mktime(datetime.now().timetuple()))
        now = datetime.fromtimestamp(timestamp)

        self.assertEqual(KayakoRequestParser._get_date(self._etree_with_data(timestamp), required=False, strict=False) , now)
        assert KayakoRequestParser._get_date(self._etree_with_data('0'), required=True) == FOREVER
        assert KayakoRequestParser._get_date(None, required=False, strict=False) == None
        assert KayakoRequestParser._get_date(self._etree_with_data(''), required=False, strict=False) == None
        assert KayakoRequestParser._get_date(self._etree_with_data('abc'), required=False, strict=False) == None

    def test__get_date_strict(self):
        import time
        from datetime import datetime
        from kayako.exception import KayakoResponseError
        from kayako.core.lib import FOREVER
        from kayako.core.object import KayakoRequestParser

        timestamp = int(time.mktime(datetime.now().timetuple()))
        now = datetime.fromtimestamp(timestamp)

        self.assertEqual(KayakoRequestParser._get_date(self._etree_with_data(timestamp), required=False, strict=True) , now)
        assert KayakoRequestParser._get_date(self._etree_with_data('0'), required=True) == FOREVER
        assert KayakoRequestParser._get_date(None, required=False, strict=True) == None
        assert KayakoRequestParser._get_date(self._etree_with_data(''), required=False, strict=True) == None
        self.assertRaises(KayakoResponseError, KayakoRequestParser._get_date, self._etree_with_data('abc'), required=False, strict=True)







# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2011, Evan Leis
#
# Distributed under the terms of the Lesser GNU General Public License (LGPL)
#-----------------------------------------------------------------------------
'''
Created on May 5, 2011

@author: evan
'''
from kayako.core.lib import ParameterObject, NodeParser, UnsetParameter
from kayako.exception import KayakoMethodNotImplementedError, KayakoRequestError, KayakoResponseError

class KayakoRequestParser(NodeParser):
    ''' 
    Overrides NodeParser methods to raise KayakoResponseErrors instead of 
    ValueError, AttributeError, and TypeError when parsing nodes/data.
    '''

    @staticmethod
    def _parse_int(data, required=True, strict=True):
        try:
            return NodeParser._parse_int(data, required=required, strict=strict)
        except Exception, error:
            raise KayakoResponseError('There was an error parsing the response (_parse_int(%s, required=%s, strict=%s):\n\t%s' % (data, required, strict, error))

    @staticmethod
    def _parse_date(data, required=True, strict=True):
        try:
            return NodeParser._parse_date(data, required=required, strict=strict)
        except Exception, error:
            raise KayakoResponseError('There was an error parsing the response (_parse_date(%s, required=%s, strict=%s):\n\t%s' % (data, required, strict, error))

    @staticmethod
    def _get_int(node, required=True, strict=True):
        try:
            return NodeParser._get_int(node, required=required, strict=strict)
        except Exception, error:
            raise KayakoResponseError('There was an error parsing the response (_get_int(%s, required=%s, strict=%s):\n\t%s' % (node, required, strict, error))

    @staticmethod
    def _get_boolean(node, required=True, strict=True):
        try:
            return NodeParser._get_boolean(node, required=required, strict=strict)
        except Exception, error:
            raise KayakoResponseError('There was an error parsing the response (_get_boolean(%s, required=%s, strict=%s):\n\t%s' % (node, required, strict, error))

    @staticmethod
    def _get_date(node, required=True, strict=True):
        try:
            return NodeParser._get_date(node, required=required, strict=strict)
        except Exception, error:
            raise KayakoResponseError('There was an error parsing the response (_get_date(%s, required=%s, strict=%s):\n\t%s' % (node, required, strict, error))

    def __str__(self):
        return '<KayakoRequestParser at %s>' % (hex(id(self)))

class KayakoObject(ParameterObject, KayakoRequestParser):
    ''' Kayako Object class meant to built from a factory. '''

    id = UnsetParameter
    api = None
    controller = None

    __required_add_parameters__ = []
    ''' Parameters required to add this object. '''
    __add_parameters__ = []
    ''' Parameters sent when adding this object. '''

    __required_save_parameters__ = []
    ''' Parameters required to save this object. '''
    __save_parameters__ = []
    ''' Parameters sent when saving this object. '''

    def __init__(self, api, **parameters):
        ParameterObject.__init__(self, **parameters)
        self.api = api

    ## ParameterObject

    @property
    def add_parameters(self):
        '''
        Return a dictionary of this objects add parameters.
        '''
        return self._parameters_from_list(self.__add_parameters__)

    @property
    def save_parameters(self):
        '''
        Return a dictionary of this objects save parameters.
        '''
        return self._parameters_from_list(self.__save_parameters__)

    @property
    def parameters(self):
        '''
        Return a dictionary of this objects save parameters.
        '''
        return self._parameters_from_list(self.__parameters__)

    ## Persistence Layer

    @classmethod
    def get_all(cls, api, *args, **kwargs):
        ''' Get all instances of this object from Kayako. '''
        raise KayakoMethodNotImplementedError('GET ALL %s is not implemented for this object.' % cls.__name__)

    @classmethod
    def get(cls, api, *args):
        ''' Get an instance of this object by ID '''
        raise KayakoMethodNotImplementedError('GET %s not implemented (id:%s)' % (cls.__name__, id))

    def _add(self, controller):
        '''
        Refactored method to check required parameters before adding. 
        Also checks self.id is an UnsetParameter.
        Returns the response returned by the API call.
        '''
        if self.id is not UnsetParameter:
            raise KayakoRequestError('Cannot add a pre-existing %s. Use save instead. (id: %s)' % (self.__class__.__name__, self.id))
        parameters = self.add_parameters
        for required_parameter in self.__required_add_parameters__:
            if required_parameter not in parameters:
                raise KayakoRequestError('Cannot add %s: Missing required field: %s.' % (self.__class__.__name__, required_parameter))
        return self.api._request(controller, 'POST', **parameters)

    def add(self):
        ''' Add a new object to Kayako '''
        raise KayakoMethodNotImplementedError('POST %s not implemented' % (self.__class__.__name__))

    def _save(self, controller, *required_parameters):
        '''
        Refactored method to check required parameters before saving. Also checks
        that 'id,' if present, is an UnsetParameter.
        Returns the response returned by the API call.
        '''
        if self.id is None or self.id is UnsetParameter:
            raise KayakoRequestError('Cannot save a non-existent %s. Use add instead.' % self.__class__.__name__)
        parameters = self.save_parameters
        for required_parameter in self.__required_save_parameters__:
            if required_parameter not in parameters:
                raise KayakoRequestError('Cannot save %s: Missing required field: %s. (id: %s)' % (self.__class__.__name__, required_parameter, self.id))
        return self.api._request(controller, 'PUT', **parameters)

    def save(self):
        ''' Save an existing object to Kayako '''
        raise KayakoMethodNotImplementedError('PUT %s not implemented' % (self.__class__.__name__))

    def _delete(self, controller):
        '''
        Refactored method to delete an object from Kayako.
        '''
        if self.id is UnsetParameter:
            raise KayakoRequestError('Cannot delete a non-existent %s. The ID of the %s to delete has not been specified.' % (self.__class__.__name__, self.__class__.__name__))
        self.api._request(controller, 'DELETE')
        self.id = UnsetParameter

    def delete(self):
        ''' Delete an object from Kayako '''
        raise KayakoMethodNotImplementedError('DELETE %s not implemented' % (self.__class__.__name__))

    def __str__(self):
        return '<%s at %s>' % (self.__class__.__name__, hex(id(self)))

    def __repr__(self):
        params = self.parameters
        return '%s(%s%s)' % (self.__class__.__name__, self.api, ', **%s' % params if params else '')

